using System;
using System.Text.RegularExpressions;

namespace SecNumber
{
	public struct SocialSecurityNumber
	{
		private enum NumberTypes { Short, Long, Invalid};
		
		private string secNumber;
		private NumberTypes numberType;

		private static bool IsExactMatch(string value, string pattern) 
		{
			Match m = Regex.Match(value, pattern);
			return(m.Success && m.Index == 0 && m.Length == value.Length);
		}

		private static NumberTypes CheckFormat(string number) 
		{
			
			bool isShortPatternUsed = IsExactMatch(number, @"\d{6}-\d{4}");
			
			if ( !( isShortPatternUsed || IsExactMatch(number, @"\d{8}-\d{4}")) )
				return NumberTypes.Invalid;
			else
				return (isShortPatternUsed?NumberTypes.Short:NumberTypes.Long);
		}

		private static void CheckDate(string number, ref NumberTypes numberType) 
		{
			bool isShortPatternUsed = (numberType == NumberTypes.Short);
			
			string datePartOfNumber = number.Substring(0, (isShortPatternUsed?6:8));
			
			try 
			{
				DateTime.ParseExact(datePartOfNumber, (isShortPatternUsed?"yyMMdd":"yyyyMMdd"), null);
			}
			catch 
			{
				numberType = NumberTypes.Invalid;
			}
		
			return;
		}

		private static void ValidateCheckSum(string number, ref NumberTypes numberType) 
		{
			
			if ( numberType == NumberTypes.Long)number = number.Substring(2);
			
			number = number.Replace("-", string.Empty);

			int sumOfNumbers = 0;
			bool doubleIt = true;
			foreach(char c in number) 
			{
				int workNum = int.Parse(c.ToString()) * (doubleIt?2:1);
				if(workNum > 9) workNum -= 9;
				sumOfNumbers += workNum;

				doubleIt = !doubleIt;
			}
			
			
			if ( (sumOfNumbers % 10) != 0 )
				numberType = NumberTypes.Invalid;
		}
		
		private static SocialSecurityNumber Parse(string Number, bool throwOnError) 
		{
			NumberTypes numberType = NumberTypes.Invalid;

			if( (numberType = CheckFormat(Number)) == NumberTypes.Invalid ) {
				if(throwOnError){throw new FormatException("String is not a correctly formatted social security number");}
			}
			else
			{
					CheckDate(Number, ref numberType);
				if ( numberType == NumberTypes.Invalid) {
					if(throwOnError){throw new FormatException("Social securiyt number doesn't contain a valid date");}
				}						
				else {
					ValidateCheckSum(Number, ref numberType);
					if ( numberType == NumberTypes.Invalid)
						if(throwOnError){throw new FormatException("String is not a valid social security number");}
				}
				
			}

			return new SocialSecurityNumber(Number, numberType);

		}

		public static SocialSecurityNumber TryParse(string Number) 
		{
			return Parse(Number, false);
		}
		


		public static SocialSecurityNumber Parse(string Number) 
		{
			return Parse(Number, true);
		}
        

		private SocialSecurityNumber(string number, NumberTypes numberType) 
		{
			this.secNumber = number;
			this.numberType = numberType;
		}
		
		public override string ToString()
		{
			return this.secNumber;
		}


		public bool IsValid
		{
			get { return this.numberType != NumberTypes.Invalid; }
		}
	}
}
